from distutils.version import LooseVersion

from zope.interface import implements
from zope.component import getAdapters
from zope.viewlet.interfaces import IViewlet
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.viewletmanager.manager import OrderedViewletManager

from interfaces import ICheatSheetsView, ICheatsManager

class VersionedViewletManager(OrderedViewletManager):
    manager_template = ViewPageTemplateFile('manage-viewletmanager.pt')
       
    def get_cheatsheets_view(self):
        p = self.__parent__
        while not p.__provides__(ICheatSheetsView):
            p = p.__parent__
        return p
       
    def filter(self, viewlets):
        """ filter out viewlets with version besides selected one """
        # filter hidden viewlets
        viewlets = super(VersionedViewletManager, self).filter(viewlets)

        cs = self.get_cheatsheets_view()
        all_versions = cs.get_all_versions()
        
        # get requested version or the last one
        show_version = self.request.get('v', all_versions[-1])
        
        # make sure requested version is correct
        if not show_version in all_versions:
            show_version = all_versions[-1]
        
        is_in = lambda value, inf, sup: inf <= value <= sup
        
        return (v for v in viewlets if is_in(show_version, *cs.get_viewlet_versions(*v)))
  
    def fix_url(self, url):
        return url.replace('/@@manage-viewlets', '/@@cs/@@manage-viewlets')

class CheatSheetsView(BrowserView):
    implements(ICheatSheetsView)
    
    # simple caches
    _v_all_versions = [] # a list of LooseVersion objects
    _v_viewlets_versions = {} # keys are viewlets names, values are veriosn tuples

    def get_viewlets(self, iface=ICheatsManager):
        """ Return all viewlets reistered for cheat viewlets managers """
        viewlets = []
        managers = getAdapters((self.context, self.request, self), iface)
        
        for name, manager in managers:
            v = getAdapters((manager.context, manager.request, manager.__parent__, manager), IViewlet)
            viewlets.extend(list(v))
        
        return viewlets
    
    def get_all_versions(self, viewlets=[]):
        """ return all versions for list of viewlets """
        if not self._v_all_versions:
            if not viewlets:
                viewlets = self.get_viewlets()

            result = []
            
            # skip versions min and max
            skip = [LooseVersion('0'), LooseVersion('*')]
            
            for name, viewlet in viewlets:
                for version in self.get_viewlet_versions(name, viewlet):
                    if version not in skip and version not in result:
                        result.append(version)
                
            result.sort()
            
            # cache result
            self._v_all_versions = result

        return self._v_all_versions

    def get_viewlet_versions(self, name, viewlet):
        """ return tuple of lower and upper version for viewlet
            if viewlet has no upper version return * """
        if not name in self._v_viewlets_versions:
            # if viewlet has no versions, assume it belongs to any version
            versions = getattr(viewlet, 'versions', '0,*')

            if not ',' in versions:
                lower, upper = versions, '*'
            else:
                lower, upper = versions.split(',')
                
            # if lower is any (*) convert it to 0, so it'll be smaller than any other version
            if lower == '*':
                lower = '0'
                    
            # convert strings to LooseVersions, and cache result
            self._v_viewlets_versions[name] = (LooseVersion(lower), LooseVersion(upper))
        
        return self._v_viewlets_versions[name]
    
    absolute_url = lambda self: self.context.absolute_url() + '/@@cs'
    getCurrentSkinName = lambda self: self.context.getCurrentSkinName()
    