// get width of column as a blueprint css class name, eg. for "span-14" return 14
get_column_width = function(obj) {
    var re = /span-(\d+)/;
    return parseInt(re.exec(obj.attr('class'))[1]);
};

// make all columns sortable
enable_sorting = function() {
    $('.column').sortable({
        connectWith: ['.column'],
        cursor: 'pointer',
        helper: function(e, o) {
            w = $(o).width();
            hdr = $(o).find('.header');
            t = hdr.text();
            h = hdr.height() + 20;
            return $('<div class="helper" style="width:'+w+'px; height:'+h+'px;">'+t+'</div>'); },
        items: '.cheat',
        placeholder: 'placeholder',
        tolerance: 'intersect'
    });
};

// turn off sorting in columns
disable_sorting = function() {
    $('.column').sortable('destroy');
};

// turn on buttons to hide / show each cheat
enable_hiding = function() {
    var show_cheat = $('#templates .show-cheat').clone(true);
    var hide_cheat = $('#templates .hide-cheat').clone(true);

    $(".cheat").each(function() {
        if($(this).hasClass("hidden")) {
            $(this).removeClass("hidden").addClass("half-hidden").find("th").append(show_cheat.clone(true));
        }
        else {
            $(this).find("th").append(hide_cheat.clone(true));
        }
    });
};

// turn off hiding cheats, replace each half-hidden cheat to completly hidden
disable_hiding = function() {
    $('.cheat .show-cheat,.cheat .hide-cheat').remove();
    $('.cheat .half-hidden').removeClass('half-hidden').addClass('hidden');
};

// serialize existing layout to json data
serialize = function() {
    var res = new Array();
    $('.column').each(function() {
        var col = "{'width':" + get_column_width($(this)) + ", 'cheats':[";
        var cheats = new Array();
        $(this).find('.cheat').each(function() {
            cheats.push("{'hide':" + $(this).hasClass('half-hidden').toString() + ", 'id':'" + $(this).attr('id') +"'}");
        });
        col += cheats.join(",") + "]}";
        res.push(col);
    });
    return "[" + res.join(",") + "]";
};

// destroy default layout and recreate it based on saved json data
deserialize = function(data) {
    // get all cheats
    var cheats = $('.cheat').clone();
    var column_manage = $('.column-manage-template').clone(true).attr('class', 'column-manage');
            
    // remove all columns
    $('.column').remove();
    
    // and recreate them
    var columns = eval(data);
    $.each(columns, function() {
        var column = $('<div class="column span-' + this.width + '"></div>');
        column.append(column_manage.clone(true));
        
        $.each(this.cheats, function() {
            var id = this.id;
            var cheat = $.grep(cheats, function(n) {
                return $(n).attr('id') == id;
            });

            cheat = $(cheat);
            if(this.hide) {
                cheat.addClass('hidden');
            }
            column.append(cheat);
        });
        
        $('.container .footer').before(column);
    });
};

// refresh table of contents, remove existing entries and recreate only visible boxes
refresh_toc = function(columns) {
    if(!columns) columns = 2;
    
    // keep first and remove other rows
    var toc_first_row = $('#cheat-toc table tr:first').clone();
    toc_first_row.find('th').attr({colspan: columns, colSpan: columns});
    $('#cheat-toc table tr').remove();

    // get all cheats except toc
    var cheats = $('.cheat:visible').not('#cheat-toc').get();
    cheats.sort(function(a, b) {
        return $(a).find('.header:first').text() > $(b).find('.header:first').text();
    });
 
    var cl = '';
    var rows = $('<tbody></tbody>').append(toc_first_row).append('<tr></tr>');
                
    // for each cheat add link to toc
    $.each(cheats, function(index, cheat) {
        cheat = $(cheat);
        var name = cheat.find('a[name]:first').attr('name'); 
        var title = cheat.find('.header:first').text();
        var a = $('<a></a>').attr('href', '#'+name).text(title);
        var td = $('<td></td>').html(a);
        var last_row = rows.find('tr:last');
        
        // put few entries in each row to save space
        if(last_row.find('td').length == columns) { 
            cl = cl == '' ? 'even' : '';
            var tr = $('<tr></tr>').addClass(cl).append(td);
            rows.append(tr);
        }
        else {
            last_row.append(td);
        };
    });
    
    // align last row to the right
    var last_row = rows.find('tr:last');
    last_row.find('td:last').attr('colspan', 1 + columns - last_row.find('td').length);
    
    $('#cheat-toc table').html(rows);
};

show_message = function(msg) {
    $('.message').hide();
    $(msg).fadeIn().click(function(){
        $(this).fadeOut();
    });
};

copy_manage_template = function() {
    $('.column').prepend($('.column-manage-template').clone(true).attr('class', 'column-manage')); 
};

get_cookie_name = function() {
    var version = $('#selected-version').text();
    return 'cheatsheets-' + version;
};

// load saved data from cookie and reorganize content
initialize_cheats = function() {

    if(document.cookie.length>0) {
        var cookie_name = get_cookie_name();
        var start = document.cookie.indexOf(cookie_name + '=');
        if(start != -1) { 
            start = start + cookie_name.length + 1;
            end = document.cookie.indexOf(";", start);
            if(end == -1) end = document.cookie.length;
            
            var data = unescape(document.cookie.substring(start, end));
            deserialize(data);
        }
        else {
            copy_manage_template();
        }
    }
    else {
        copy_manage_template();
    };

    // actions
    
    $('.wider').click(function(e) {
        e.preventDefault();
        var p = $(this).parents('.column');
        
        var width = get_column_width(p);
        p.removeClass('span-'+width).addClass('span-'+(width+1));
    });

    $('.narrower').click(function(e) {
        e.preventDefault();
        var p = $(this).parents('.column');
        
        var width = get_column_width(p);
        if(width > 3) {
            p.removeClass('span-'+width).addClass('span-'+(width-1));
        }
    });

    $('.add').click(function(e) {
        e.preventDefault();
        var p = $(this).parents('.column');
        
        // make new column and append it after this column
        var new_column = $('<div class="column span-8">&nbsp;</div>');
        var column_manage = $('.column-manage-template').clone(true).attr('class', 'column-manage').show();
        new_column.append(column_manage);
        p.after(new_column);
               
        // reload sorting
        enable_sorting();
    });

    $('.remove').click(function(e) {
        e.preventDefault();
        var p = $(this).parents('.column');
        
        if($('.column').length > 1) {
            // move cheats from this column to first one
            var cheats = p.find('.cheat').clone();
            $('.column:first').append(cheats);

            // remove column
            p.remove();
        }
    });        

    $('#templates .show-cheat').click(function(e) {
        e.preventDefault();
        // remove class half-hidden, add button hide and remove self
        $(this).parents('.cheat').removeClass('half-hidden');
        $(this).after($('#templates .hide-cheat').clone(true));
        $(this).remove();
    });

    $('#templates .hide-cheat').click(function(e) {
        e.preventDefault();
        // add class half-hidden, add button show and remove self
        $(this).parents('.cheat').addClass('half-hidden');
        $(this).after($('#templates .show-cheat').clone(true));
        $(this).remove();
    });

    $('#customize').click(function(e) {
        e.preventDefault();
        
        enable_sorting();
        enable_hiding();
        
        $('.column-manage,#save,#cancel,#manage').show();
        $(this).hide();
        
        show_message('#message_customize');
    });

    $('#save').click(function(e) {
        e.preventDefault();
        
        disable_sorting();   
        disable_hiding();
        
        // save changes - seriazlize them to json and set cookie
        var data = serialize();
        var expires = new Date();
        expires.setDate(expires.getDate() + 365);
        document.cookie = get_cookie_name() + '=' + escape(data) + ';expires=' + expires.toGMTString();

        $('#customize').show();
        $('.column-manage,#cancel,#manage').hide(); $(this).hide();
        
        show_message('#message_save');
    });
    
    $('#cancel').click(function(e) {
        e.preventDefault();
        
        disable_sorting();   
        disable_hiding();
        
        $('#customize').show();
        $('.column-manage,#save,#manage').hide(); $(this).hide();
        
        show_message('#message_cancel');
    });
};
